@@ -678,52 +678,61 @@ impl FitSessionMsg {
     }
 }
 
+// Auto-generated by print_message_struct in lib.rs
 pub struct FitDeviceInfoMsg {
-    pub timestamp: Option<u32>, // 1 * s + 0,
-    pub serial_number: Option<u32>, //
-    pub cum_operating_time: Option<u32>, // 1 * s + 0, Reset by new battery or charge.
-    pub product_name: Option<String>, // Optional free form string to indicate the devices name or model
-    pub manufacturer: Option<u16>, //
+    pub battery_voltage: Option<u16>,
+    pub cum_operating_time: Option<u32>,
+    pub serial_number: Option<u32>,
     pub product: Option<u16>,
+    pub timestamp: Option<u32>,
+    pub sensor_position: Option<u8>,
+    pub source_type: Option<u8>,
     pub software_version: Option<u16>,
-    pub battery_voltage: Option<u16>, // 256 * V + 0,
-    pub ant_device_number: Option<u16>, //
-    pub device_index: Option<u8>, //
-    pub device_type: Option<u8>, //
-    pub hardware_version: Option<u8>, //
-    pub battery_status: Option<u8>, //
-    //FIT_BODY_LOCATION sensor_position, // Indicates the location of the sensor
-    pub descriptor: Option<String>, // Used to describe the sensor or location
-    pub ant_transmission_type: Option<u8>, //
-    //FIT_ANT_NETWORK ant_network, //
-    pub source_type: Option<u8> //
+    pub ant_transmission_type: Option<u8>,
+    pub ant_device_number: Option<u16>,
+    pub descriptor: Option<String>,
+    pub device_type: Option<u8>,
+    pub ant_network: Option<u8>,
+    pub product_name: Option<String>,
+    pub device_index: Option<u8>,
+    pub hardware_version: Option<u8>,
+    pub battery_status: Option<u8>,
+    pub manufacturer: Option<u16>,
 }
 
 impl FitDeviceInfoMsg {
 
     /// Constructor: Takes the fields that were read by the file parser and puts them into a structure.
     pub fn new(fields: Vec<FitFieldValue>) -> Self {
-        let mut msg = FitDeviceInfoMsg{ timestamp: None, serial_number: None, cum_operating_time: None, product_name: None, manufacturer: None,
-            product: None, software_version: None, battery_voltage: None, ant_device_number: None, device_index: None, device_type: None,
-            battery_status: None, hardware_version: None, descriptor: None, ant_transmission_type: None, source_type: None };
+        let mut msg = FitDeviceInfoMsg { battery_voltage: None, 
+            cum_operating_time: None, serial_number: None, product: None,
+            timestamp: None, sensor_position: None, source_type: None, 
+            software_version: None, ant_transmission_type: None, ant_device_number: None, 
+            descriptor: None, device_type: None, ant_network: None, 
+            product_name: None, device_index: None, hardware_version: None, 
+            battery_status: None, manufacturer: None, 
+        };
 
         for field in fields {
             match field.field_def {
-                0 => { msg.device_index = Some(field.get_u8()); },
-                1 => { msg.device_type = Some(field.get_u8()); },
-                2 => { msg.manufacturer = Some(field.get_u16()); },
+                10 => { msg.battery_voltage = Some(field.get_u16()); },
+                7 => { msg.cum_operating_time = Some(field.get_u32()); },
                 3 => { msg.serial_number = Some(field.get_u32()); },
                 4 => { msg.product = Some(field.get_u16()); },
-                5 => { msg.software_version = Some(field.get_u16()); },
-                6 => { msg.hardware_version = Some(field.get_u8()); },
-                7 => { msg.cum_operating_time = Some(field.get_u32()); },
-                10 => { msg.battery_voltage = Some(field.get_u16()); },
-                11 => { msg.battery_status = Some(field.get_u8()); },
-                16 => { msg.ant_device_number = Some(field.get_u16()); },
-                19 => { msg.descriptor = Some(field.string); },
-                21 => { msg.ant_device_number = Some(field.get_u16()); },
+                253 => { msg.timestamp = Some(field.get_u32()); },
+                18 => { msg.sensor_position = Some(field.get_u8()); },
                 25 => { msg.source_type = Some(field.get_u8()); },
+                5 => { msg.software_version = Some(field.get_u16()); },
+                20 => { msg.ant_transmission_type = Some(field.get_u8()); },
+                21 => { msg.ant_device_number = Some(field.get_u16()); },
+                19 => { msg.descriptor = Some(field.string); },
+                1 => { msg.device_type = Some(field.get_u8()); },
+                22 => { msg.ant_network = Some(field.get_u8()); },
                 27 => { msg.product_name = Some(field.string); },
+                0 => { msg.device_index = Some(field.get_u8()); },
+                6 => { msg.hardware_version = Some(field.get_u8()); },
+                11 => { msg.battery_status = Some(field.get_u8()); },
+                2 => { msg.manufacturer = Some(field.get_u16()); },
                 _ => { panic!("Device Info field not implemented {:#x}", field.field_def); }
             }
         }
@@ -731,90 +740,189 @@ impl FitDeviceInfoMsg {
     }
 }
 
+// Auto-generated by print_message_struct in lib.rs
 pub struct FitRecordMsg {
-    pub timestamp: Option<u32>, // 1 * s + 0,
-    pub position_lat: Option<i32>, // 1 * semicircles + 0,
-    pub position_long: Option<i32>, // 1 * semicircles + 0,
-    pub distance: Option<u32>, // 100 * m + 0,
-    pub time_from_course: Option<i32>, // 1000 * s + 0,
-    pub total_cycles: Option<u32>, // 1 * cycles + 0,
-    pub accumulated_power: Option<u32>, // 1 * watts + 0,
-    pub enhanced_speed: Option<u32>, // 1000 * m/s + 0,
-    pub enhanced_altitude: Option<u32>, // 5 * m + 500,
-    pub altitude: Option<u16>, // 5 * m + 500,
-    pub speed: Option<u16>, // 1000 * m/s + 0,
-    pub power: Option<u16>, // 1 * watts + 0,
-    pub grade: Option<i16>, // 100 * % + 0,
-    pub compressed_accumulated_power: Option<u16>, // 1 * watts + 0,
-    pub vertical_speed: Option<i16>, // 1000 * m/s + 0,
-    pub calories: Option<u16>, // 1 * kcal + 0,
-    pub vertical_oscillation: Option<u16>, // 10 * mm + 0,
-    pub stance_time_percent: Option<u16>, // 100 * percent + 0,
-    pub stance_time: Option<u16>, // 10 * ms + 0,
-    pub ball_speed: Option<u16>, // 100 * m/s + 0,
-    pub cadence256: Option<u16>, // 256 * rpm + 0, Log cadence and fractional cadence for backwards compatability
-    pub total_hemoglobin_conc: Option<u16>, // 100 * g/dL + 0, Total saturated and unsaturated hemoglobin
-    pub total_hemoglobin_conc_min: Option<u16>, // 100 * g/dL + 0, Min saturated and unsaturated hemoglobin
-    pub total_hemoglobin_conc_max: Option<u16>, // 100 * g/dL + 0, Max saturated and unsaturated hemoglobin
-    pub saturated_hemoglobin_percent: Option<u16>, // 10 * % + 0, Percentage of hemoglobin saturated with oxygen
-    pub saturated_hemoglobin_percent_min: Option<u16>, // 10 * % + 0, Min percentage of hemoglobin saturated with oxygen
-    pub saturated_hemoglobin_percent_max: Option<u16>, // 10 * % + 0, Max percentage of hemoglobin saturated with oxygen
-    pub heart_rate: Option<u8>, // 1 * bpm + 0,
-    pub cadence: Option<u8>, // 1 * rpm + 0,
-    //FIT_BYTE compressed_speed_distance[FIT_RECORD_MESG_COMPRESSED_SPEED_DISTANCE_COUNT]: Option<u8>, //
-    pub resistance: Option<u8>, // Relative. 0 is none  254 is Max.
-    pub cycle_length: Option<u8>, // 100 * m + 0,
-    pub temperature: Option<i8>, // 1 * C + 0,
-    //FIT_UINT8 speed_1s[FIT_RECORD_MESG_SPEED_1S_COUNT], // 16 * m/s + 0, Speed at 1s intervals.  Timestamp field indicates time of last array element.
-    pub cycles: Option<u8>, // 1 * cycles + 0,
-    pub left_right_balance: Option<u8>,
-    pub gps_accuracy: Option<u8>, // 1 * m + 0,
-    pub activity_type: Option<u8>,
-    pub left_torque_effectiveness: Option<u8>, // 2 * percent + 0,
-    pub right_torque_effectiveness: Option<u8>, // 2 * percent + 0,
-    pub left_pedal_smoothness: Option<u8>, // 2 * percent + 0,
-    pub right_pedal_smoothness: Option<u8>, // 2 * percent + 0,
-    pub combined_pedal_smoothness: Option<u8>, // 2 * percent + 0,
-    pub time128: Option<u8>, // 128 * s + 0,
+    pub step_length: Option<u16>,
+    pub resistance: Option<u8>,
+    pub speed: Option<u16>,
+    pub accumulated_power: Option<u32>,
+    pub next_stop_depth: Option<u32>,
     pub stroke_type: Option<u8>,
+    pub heart_rate: Option<u8>,
+    pub cycles: Option<u8>,
+    pub total_hemoglobin_conc: Option<u16>,
+    pub time_from_course: Option<i32>,
+    pub saturated_hemoglobin_percent: Option<u16>,
+    pub cadence256: Option<u16>,
+    pub ndl_time: Option<u32>,
+    pub next_stop_time: Option<u32>,
+    pub time128: Option<u8>,
+    pub left_right_balance: Option<u8>,
+    pub temperature: Option<i8>,
+    pub position_long: Option<i32>,
+    pub motor_power: Option<u16>,
+    pub vertical_ratio: Option<u16>,
+    pub grit: Option<f32>,
+    pub compressed_accumulated_power: Option<u16>,
+    pub position_lat: Option<i32>,
+    pub ebike_assist_mode: Option<u8>,
+    pub n2_load: Option<u16>,
+    pub grade: Option<i16>,
+    pub left_power_phase: Option<u8>,
+    pub power: Option<u16>,
+    pub timestamp: Option<u32>,
+    pub vertical_oscillation: Option<u16>,
+    pub fractional_cadence: Option<u8>,
+    pub saturated_hemoglobin_percent_min: Option<u16>,
+    pub cycle_length: Option<u8>,
+    pub right_pedal_smoothness: Option<u8>,
+    pub total_hemoglobin_conc_max: Option<u16>,
+    pub stance_time_percent: Option<u16>,
+    pub enhanced_altitude: Option<u32>,
+    pub stance_time_balance: Option<u16>,
+    pub speed_1s: Option<u8>,
+    pub battery_soc: Option<u8>,
+    pub total_hemoglobin_conc_min: Option<u16>,
+    pub cns_load: Option<u8>,
+    pub distance: Option<u32>,
     pub zone: Option<u8>,
-    pub fractional_cadence: Option<u8>, // 128 * rpm + 0,
-    //FIT_DEVICE_INDEX device_index;
-    pub battery_soc: Option<u8>
+    pub ball_speed: Option<u16>,
+    pub gps_accuracy: Option<u8>,
+    pub absolute_pressure: Option<u32>,
+    pub calories: Option<u16>,
+    pub stance_time: Option<u16>,
+    pub right_pco: Option<i8>,
+    pub ebike_travel_range: Option<u16>,
+    pub left_torque_effectiveness: Option<u8>,
+    pub activity_type: Option<u8>,
+    pub depth: Option<u32>,
+    pub enhanced_speed: Option<u32>,
+    pub total_cycles: Option<u32>,
+    pub vertical_speed: Option<i16>,
+    pub combined_pedal_smoothness: Option<u8>,
+    pub saturated_hemoglobin_percent_max: Option<u16>,
+    pub altitude: Option<u16>,
+    pub left_pco: Option<i8>,
+    pub left_power_phase_peak: Option<u8>,
+    pub flow: Option<f32>,
+    pub device_index: Option<u8>,
+    pub cadence: Option<u8>,
+    pub ebike_assist_level_percent: Option<u8>,
+    pub right_power_phase_peak: Option<u8>,
+    pub ebike_battery_level: Option<u8>,
+    pub compressed_speed_distance: Option<u8>,
+    pub left_pedal_smoothness: Option<u8>,
+    pub right_power_phase: Option<u8>,
+    pub right_torque_effectiveness: Option<u8>,
+    pub time_to_surface: Option<u32>,
 }
 
 impl FitRecordMsg {
 
     /// Constructor: Takes the fields that were read by the file parser and puts them into a structure.
     pub fn new(fields: Vec<FitFieldValue>) -> Self {
-
-        let mut msg = FitRecordMsg{ timestamp: None, position_lat: None, position_long: None, distance: None, time_from_course: None, total_cycles: None, accumulated_power: None,
-            enhanced_speed: None, enhanced_altitude: None, altitude: None, speed: None, power: None, grade: None, compressed_accumulated_power: None, vertical_speed: None,
-            calories: None, vertical_oscillation: None, stance_time_percent: None, stance_time: None, ball_speed: None, cadence256: None, total_hemoglobin_conc: None,
-            total_hemoglobin_conc_min: None, total_hemoglobin_conc_max: None, saturated_hemoglobin_percent: None, saturated_hemoglobin_percent_min: None,
-            saturated_hemoglobin_percent_max: None, heart_rate: None, cadence: None, resistance: None, cycle_length: None, temperature: None,
-            cycles: None, left_right_balance: None, gps_accuracy: None, activity_type: None, left_torque_effectiveness: None, right_torque_effectiveness: None,
-            left_pedal_smoothness: None, right_pedal_smoothness: None, combined_pedal_smoothness: None, time128: None, stroke_type: None, zone: None, fractional_cadence: None,
-            battery_soc: None };
+        let mut msg = FitRecordMsg { step_length: None, 
+            resistance: None, speed: None, accumulated_power: None, 
+            next_stop_depth: None, stroke_type: None, heart_rate: None, 
+            cycles: None, total_hemoglobin_conc: None, time_from_course: None, 
+            saturated_hemoglobin_percent: None, cadence256: None, ndl_time: None, 
+            next_stop_time: None, time128: None, left_right_balance: None, 
+            temperature: None, position_long: None, motor_power: None, 
+            vertical_ratio: None, grit: None, compressed_accumulated_power: None, 
+            position_lat: None, ebike_assist_mode: None, n2_load: None, 
+            grade: None, left_power_phase: None, power: None, 
+            timestamp: None, vertical_oscillation: None, fractional_cadence: None, 
+            saturated_hemoglobin_percent_min: None, cycle_length: None, right_pedal_smoothness: None, 
+            total_hemoglobin_conc_max: None, stance_time_percent: None, enhanced_altitude: None, 
+            stance_time_balance: None, speed_1s: None, battery_soc: None, 
+            total_hemoglobin_conc_min: None, cns_load: None, distance: None, 
+            zone: None, ball_speed: None, gps_accuracy: None, 
+            absolute_pressure: None, calories: None, stance_time: None, 
+            right_pco: None, ebike_travel_range: None, left_torque_effectiveness: None, 
+            activity_type: None, depth: None, enhanced_speed: None, 
+            total_cycles: None, vertical_speed: None, combined_pedal_smoothness: None, 
+            saturated_hemoglobin_percent_max: None, altitude: None, left_pco: None, 
+            left_power_phase_peak: None, flow: None, device_index: None, 
+            cadence: None, ebike_assist_level_percent: None, right_power_phase_peak: None, 
+            ebike_battery_level: None, compressed_speed_distance: None, left_pedal_smoothness: None, 
+            right_power_phase: None, right_torque_effectiveness: None, time_to_surface: None, 
+        };
 
         for field in fields {
             match field.field_def {
-                0 => { msg.position_lat = Some(field.get_i32()); },
-                1 => { msg.position_long = Some(field.get_i32()); },
-                2 => { msg.altitude = Some(field.get_u16()); },
-                3 => { msg.heart_rate = Some(field.get_u8()); },
-                4 => { msg.cadence = Some(field.get_u8()); },
-                5 => { msg.distance = Some(field.get_u32()); },
+                85 => { msg.step_length = Some(field.get_u16()); },
+                10 => { msg.resistance = Some(field.get_u8()); },
                 6 => { msg.speed = Some(field.get_u16()); },
-                7 => { msg.power = Some(field.get_u16()); },
-                9 => { msg.grade = Some(field.get_i16()); },
+                29 => { msg.accumulated_power = Some(field.get_u32()); },
+                93 => { msg.next_stop_depth = Some(field.get_u32()); },
+                49 => { msg.stroke_type = Some(field.get_u8()); },
+                3 => { msg.heart_rate = Some(field.get_u8()); },
+                18 => { msg.cycles = Some(field.get_u8()); },
+                54 => { msg.total_hemoglobin_conc = Some(field.get_u16()); },
+                11 => { msg.time_from_course = Some(field.get_i32()); },
+                57 => { msg.saturated_hemoglobin_percent = Some(field.get_u16()); },
+                52 => { msg.cadence256 = Some(field.get_u16()); },
+                96 => { msg.ndl_time = Some(field.get_u32()); },
+                94 => { msg.next_stop_time = Some(field.get_u32()); },
+                48 => { msg.time128 = Some(field.get_u8()); },
+                30 => { msg.left_right_balance = Some(field.get_u8()); },
                 13 => { msg.temperature = Some(field.get_i8()); },
-                31 => { msg.gps_accuracy = Some(field.get_u8()); },
-                43 => { msg.left_torque_effectiveness = Some(field.get_u8()); },
-                44 => { msg.right_torque_effectiveness = Some(field.get_u8()); },
-                45 => { msg.left_pedal_smoothness = Some(field.get_u8()); },
+                1 => { msg.position_long = Some(field.get_i32()); },
+                82 => { msg.motor_power = Some(field.get_u16()); },
+                83 => { msg.vertical_ratio = Some(field.get_u16()); },
+                114 => { msg.grit = Some(field.get_f32()); },
+                28 => { msg.compressed_accumulated_power = Some(field.get_u16()); },
+                0 => { msg.position_lat = Some(field.get_i32()); },
+                119 => { msg.ebike_assist_mode = Some(field.get_u8()); },
+                98 => { msg.n2_load = Some(field.get_u16()); },
+                9 => { msg.grade = Some(field.get_i16()); },
+                69 => { msg.left_power_phase = Some(field.get_u8()); },
+                7 => { msg.power = Some(field.get_u16()); },
+                253 => { msg.timestamp = Some(field.get_u32()); },
+                39 => { msg.vertical_oscillation = Some(field.get_u16()); },
+                53 => { msg.fractional_cadence = Some(field.get_u8()); },
+                58 => { msg.saturated_hemoglobin_percent_min = Some(field.get_u16()); },
+                12 => { msg.cycle_length = Some(field.get_u8()); },
                 46 => { msg.right_pedal_smoothness = Some(field.get_u8()); },
+                56 => { msg.total_hemoglobin_conc_max = Some(field.get_u16()); },
+                40 => { msg.stance_time_percent = Some(field.get_u16()); },
+                78 => { msg.enhanced_altitude = Some(field.get_u32()); },
+                84 => { msg.stance_time_balance = Some(field.get_u16()); },
+                17 => { msg.speed_1s = Some(field.get_u8()); },
                 81 => { msg.battery_soc = Some(field.get_u8()); },
+                55 => { msg.total_hemoglobin_conc_min = Some(field.get_u16()); },
+                97 => { msg.cns_load = Some(field.get_u8()); },
+                5 => { msg.distance = Some(field.get_u32()); },
+                50 => { msg.zone = Some(field.get_u8()); },
+                51 => { msg.ball_speed = Some(field.get_u16()); },
+                31 => { msg.gps_accuracy = Some(field.get_u8()); },
+                91 => { msg.absolute_pressure = Some(field.get_u32()); },
+                33 => { msg.calories = Some(field.get_u16()); },
+                41 => { msg.stance_time = Some(field.get_u16()); },
+                68 => { msg.right_pco = Some(field.get_i8()); },
+                117 => { msg.ebike_travel_range = Some(field.get_u16()); },
+                43 => { msg.left_torque_effectiveness = Some(field.get_u8()); },
+                42 => { msg.activity_type = Some(field.get_u8()); },
+                92 => { msg.depth = Some(field.get_u32()); },
+                73 => { msg.enhanced_speed = Some(field.get_u32()); },
+                19 => { msg.total_cycles = Some(field.get_u32()); },
+                32 => { msg.vertical_speed = Some(field.get_i16()); },
+                47 => { msg.combined_pedal_smoothness = Some(field.get_u8()); },
+                59 => { msg.saturated_hemoglobin_percent_max = Some(field.get_u16()); },
+                2 => { msg.altitude = Some(field.get_u16()); },
+                67 => { msg.left_pco = Some(field.get_i8()); },
+                70 => { msg.left_power_phase_peak = Some(field.get_u8()); },
+                115 => { msg.flow = Some(field.get_f32()); },
+                62 => { msg.device_index = Some(field.get_u8()); },
+                4 => { msg.cadence = Some(field.get_u8()); },
+                120 => { msg.ebike_assist_level_percent = Some(field.get_u8()); },
+                72 => { msg.right_power_phase_peak = Some(field.get_u8()); },
+                118 => { msg.ebike_battery_level = Some(field.get_u8()); },
+                8 => { msg.compressed_speed_distance = Some(field.get_u8()); },
+                45 => { msg.left_pedal_smoothness = Some(field.get_u8()); },
+                71 => { msg.right_power_phase = Some(field.get_u8()); },
+                44 => { msg.right_torque_effectiveness = Some(field.get_u8()); },
+                95 => { msg.time_to_surface = Some(field.get_u32()); },
                 _ => { panic!("Record field not implemented {:#x}", field.field_def); }
             }
         }
@@ -878,6 +986,14 @@ impl FitFieldValue {
     pub fn get_u64(&self) -> u64 {
         return self.num_uint as u64;
     }
+
+    pub fn get_f32(&self) -> f32 {
+        return self.num_float as f32;
+    }
+
+    pub fn get_f64(&self) -> f64 {
+        return self.num_float as f64;
+    }
 }
 
 /// Encapsulates a custom field definition, as described by definition messages and used by data messages.
